import models
import unittest
import random
from mock import patch

class TestBoard(unittest.TestCase):
    def setUp(self):
        self.board = models.import_board_data('./board_graph.json')

    def test_border_countries(self):
        country_A = self.board.countries['alaska']
        country_B = self.board.countries['northwest territory']
        self.assertIn(country_A, country_B.border_countries)
        self.assertIn(country_B, country_A.border_countries)

    def test_all_borders(self):
        for country_A in self.board.countries:
            for country_B in self.board.countries[country_A].border_countries:
                self.assertIn(self.board.countries[country_A], country_B.border_countries)
                self.assertIn(country_B, self.board.countries[country_A].border_countries)

    def test_not_border_countries(self):
        country_A = self.board.countries['iceland']
        country_B = self.board.countries['northwest territory']
        self.assertNotIn(country_A, country_B.border_countries)
        self.assertNotIn(country_B, country_A.border_countries)

    def test_deploy_troops(self):
        country_A = self.board.countries['iceland']
        erty = models.Player('Erty')
        erty.choose_country(country_A)
        country_A.troops = 20
        erty.deploy_troops(country_A, 10)
        self.assertEqual(country_A.troops, 30)

    def test_deploy_troops_to_wrong_country(self):
        country_A = self.board.countries['northwest territory']
        erty = models.Player('Erty')
        alex = models.Player('Alex')
        erty.choose_country(country_A)
        country_A.troops = 21
        with self.assertRaises(AssertionError):
            alex.deploy_troops(country_A, 10)

    def test_attack(self):
        country_A = self.board.countries['alaska']
        country_B = self.board.countries['northwest territory']
        players = [models.Player('Erty'),models.Player('Alex')]
        country_A.owner = players[0]
        country_B.owner = players[1]
        country_A.troops = 30
        country_B.troops = 10
        with patch.object(random, 'randint') as mock_method:
            mock_method.side_effect = [6,6,1,1,1]
            country_A.attack(country_B, 3, 4)
            self.assertEqual(country_A.troops, 28)
            self.assertEqual(country_B.troops, 10)


        with patch.object(random, 'randint') as mock_method:
            mock_method.side_effect = [1,1,6,6,6]
            country_A.attack(country_B,3, 2)
            self.assertEqual(country_A.troops, 28)
            self.assertEqual(country_B.troops, 8)

    def test_attack_wrong_country(self):
        country_A = self.board.countries['alaska']
        country_B = self.board.countries['japan']
        country_A.troops, country_B.troops = 15, 15
        with self.assertRaises(AssertionError):
            country_A.attack(country_B, 3, 1)


    def test_attack_self(self):
        country_A = self.board.countries['alaska']
        country_B = self.board.countries['northwest territory']
        alex = models.Player('Alex')
        alex.choose_country(country_A)
        alex.choose_country(country_B)
        with self.assertRaises(AssertionError):
            country_A.attack(country_B, 3, 1)

    def test_continent_bonus(self):
        alex = models.Player('Alex')
        north_america = self.board.continents["north america"]
        for country in north_america.countries.values():
            alex.choose_country(country)
        self.assertEqual(north_america.get_player_set(), {alex})
        self.assertEqual(north_america.bonus, 5)

    def test_is_card_set(self):
        #this is a really dumb test, but it sort of works and I'm too lazy to make it better right now
        card1 = self.cards[0]
        card2 = self.cards[1]
        card3 = self.cards[2]
        card4 = self.cards[3]
        card5 = self.cards[4]
        set1=card1.is_set_with(card2, card3)
        set2=card1.is_set_with(card2, card4)
        set3=card1.is_set_with(card2, card5)
        set4=card1.is_set_with(card3, card4)
        set5=card1.is_set_with(card3, card5)
        set6=card1.is_set_with(card4, card5)
        set7=card2.is_set_with(card3, card4)
        set8=card2.is_set_with(card3, card5)
        set9=card2.is_set_with(card4, card5)
        set10=card3.is_set_with(card4, card5)
        self.assertEqual(set1 or set2 or set3 or set4 or set5 or set6 or set7 or set8 or set9 or set10, True)

    def test_has_card_set(self):
        card1 = self.cards[0]
        card2 = self.cards[1]
        card3 = self.cards[2]
        card4 = self.cards[3]
        card5 = self.cards[4]
        alex = models.Player("Alex")
        alex.cards = {card1, card2, card3, card4, card5}
        self.assertEqual(alex.has_card_set(), True)

    def test_not_card_set(self):
        card1 = self.cards[0]
        card2 = self.cards[1]
        card3 = self.cards[2]
        alex = models.Player("Alex")
        alex.cards = { card2, card3}
        self.assertEqual(alex.has_card_set(), False)

if __name__ == "__main__":
    unittest.main()


from uuid import uuid4
import random
import math
import json
import models

initial_troops = {3: 35,
                  4: 30,
                  5: 25,
                  6: 20}


class Game(object):

    def __init__(self, players):
        self.board = models.import_board_data()
        self.card_deck = self.board.cards.values()
        self.players = players
        random.shuffle(self.card_deck)
        self.uid = uuid4()
        self.init_turn = 0
        self.init_turn = len(self.board.countries) + initial_troops[len(self.players)]
        self.turn = 0
        self.max_turns = 1000
        self.broadcast_count = 0
        self.card_sets_traded_in = 0
        self.winner = None
        self.last_action = ''

    def start_game(self):
        #assign countries to players
        self.players.start_game()
        self.init_deploy()
        self.play_game()

    def init_deploy(self):

        print "started deployment"

        troops_to_deploy = initial_troops[len(self.players)]*len(self.players)

        while {c for c in self.board.countries.values() if not c.owner}:
            self.players.next()
            self.init_turn += 1
            self.players.choose_country(self)
            troops_to_deploy -= 1

        print "countries choosen"
        print {p.name:[c.name for c in p.countries] for p in self.players}

        for _ in xrange(troops_to_deploy):
            self.players.next()
            self.init_turn += 1
            self.players.current_player.troops_to_deploy = 1
            self.players.deploy_troops(self)

        print "initial troops deployed"

    def play_game(self):

        print "starting game"

        self.players.restart()
        while not self.check_for_winner():
            self.turn += 1
            print "starting turn %s - player %s's turn" % (self.turn, self.players.current_player.name)
            print {p.name:{'countries':len(p.countries),'total troops':sum([c.troops for c in p.countries])} for p in self.players}
            self.players.next()
            self.deployment_phase()
            self.players.attack(self)
            self.players.reinforce(self)
            if(self.players.current_player.earned_card_this_turn and self.card_deck):
                self.players.current_player.earned_card_this_turn = False
                self.players.current_player.cards.add(self.card_deck.pop())

    def deployment_phase(self):
        self.phase = 'deployment'

        self.players.current_player.troops_to_deploy += max(int(math.ceil(len(self.players.current_player.countries)/3.)), 3)
        self.players.current_player.troops_to_deploy += sum({con.bonus for con in self.board.continents.values()
                                                             if con.get_player_set() == {self.players.current_player}})
        self.players.spend_cards(self)
        self.players.deploy_troops(self)

    def attack(self, attacking_country, defending_country, attacking_troops, moving_troops):
        self.phase = 'attacking'
        assert attacking_country.owner == self.players.current_player
        defending_country_player = defending_country.owner
        country_invaded = attacking_country.attack(defending_country, attacking_troops, moving_troops)
        if country_invaded and not self.players.current_player.earned_card_this_turn:
            self.players.current_player.earned_card_this_turn = True
        if not defending_country_player.countries:
            self.eliminate_player(self.players.current_player, defending_country_player)
            if len(self.players.current_player.cards) >= 5:
                self.players.force_cards_spend(self)
        return country_invaded

    def reinforce(self, origin_country, destination_country, troops):
        assert origin_country.owner == self.players.current_player
        assert destination_country.owner == self.players.current_player
        assert origin_country.troops - troops >= 1
        if not origin_country:
            return True
        destination_country.add_troops(self.players.current_player, troops)
        origin_country.troops -= troops

    def check_for_winner(self):
        players_remaining = {p for p in self.players if not p.is_eliminated}
        neutral_players = {p for p in players_remaining if p.is_neutral}
        if len(players_remaining) == 1:
            self.winner = list(players_remaining)[0]
            print "Player %s DOMINATES!" % self.winner.name
            return True
        elif players_remaining == neutral_players:
            self.winner = None
            print "All players quit. DRAW"
            return True
        elif self.turn == self.max_turns:
            self.winner = None
            print "Reached 1000 turns. DRAW"
            return True
        else:
            return False

    def eliminate_player(self, eliminator, eliminated):
        assert eliminator is not None
        assert eliminated is not None
        assert eliminator.is_eliminated is False
        assert eliminated.is_eliminated is False
        assert eliminator.is_neutral is False
        assert len(eliminated.countries) is 0

        eliminator.cards = eliminator.cards.union(eliminated.cards)
        eliminated.cards = set()
        eliminated.is_eliminated = True

    def get_troops_for_card_set(self, cards):
        assert len(cards) == 3
        for card in cards:
            assert card in self.players.current_player.cards
        assert cards[0].is_set_with(cards[1], cards[2])
        self.players.current_player.cards -= set(cards)
        for card in [c for c in cards if c.value != 'wild']:
            if self.board.countries[card.country_name] in self.players.current_player.countries:
                self.board.countries[card.country_name].troops += 2
                break
        if(self.card_sets_traded_in < 6):
            self.card_sets_traded_in += 1
            return (self.card_sets_traded_in-1 + 2) * 2
        else:
            self.card_sets_traded_in += 1
            return (self.card_sets_traded_in - 3) * 5

    def game_state_json(self, player):
        """
        Returns game state as JSON for sending to clients.
        Where g = Game(*args),
        g.game_state_json(player)
        returns a JSON object of the entire game state as visible to player,
        where player is the requesting player
        """
        game_state = {'game':{},'you':{}}
        game_state['game']['countries'] = {}
        for key in self.board.countries:
            country = self.board.countries[key]
            game_state['game']['countries'][key] = country
        game_state['game']['players'] = self.players
        game_state['you'] = player
        game_state['game']['turn'] = self.turn
        game_state['game']['uid'] = str(self.uid)
        game_state['game']['last_action'] = self.last_action
        game_state['game']['broadcast_count'] = self.broadcast_count
        return json.dumps(game_state, cls=GameEncoder)


class GameEncoder(json.JSONEncoder):
    """Special JSON encoder for our objects"""
    def default(self, obj):
        if isinstance(obj, models.Country):
            if obj.owner is None:
                return {'owner': 'none',
                        'troops': obj.troops,
                        }
            else:
                return {'owner': obj.owner.name,
                        'troops': obj.troops,
                        }

        elif isinstance(obj, models.Player):
            return {'name': obj.name,
                    'is_eliminated': obj.is_eliminated,
                    'cards': list(obj.cards),
                    'earned_cards_this_turn': obj.earned_card_this_turn,
                    'countries': [country.name for country in obj.countries],
                    'troops_to_deploy': obj.troops_to_deploy,
                    'available_actions': obj.available_actions,
                    }
        elif isinstance(obj, models.Players):
            return {p.name:{'is_eliminated': p.is_eliminated,
                            'cards': len(p.cards),
                            'is_neutral': p.is_neutral,
                            }
                    for p in obj.players_list}
        elif isinstance(obj, models.Continent):
            return {'countries': obj.countries, 'bonus': obj.bonus}

        elif isinstance(obj, models.Card):
            return {'country_name':obj.country_name, 'value': obj.value}

        else:
            return json.JSONEncoder.default(self, obj)



import random
import json
import itertools
import os

class Country(object):
    def __init__(self, name, border_countries):
        self.border_countries = border_countries
        self.name = name
        self.owner = None
        self.troops = 0

    def attack(self, country, attacking_troops, moving_troops):
        assert country in self.border_countries
        assert country.owner is not None
        assert country.owner is not self.owner
        assert self.troops - (attacking_troops + moving_troops) >= 1
        assert attacking_troops > 0
        assert attacking_troops <= 3

        if country.troops >= 2:
            defending_die = 2
        elif country.troops == 1:
            defending_die = 1
        else:
            raise NameError('defending country has no troops')

        if attacking_troops >= 3:
            attacking_die = 3
        elif attacking_troops == 2:
            attacking_die = 2
        elif attacking_troops == 1:
            attacking_die = 1
        else:
            raise NameError('attacking country has no troops')

        defending_rolls = sorted([random.randint(1, 6) for i in range(defending_die)],
                                 reverse=True)
        attacking_rolls = sorted([random.randint(1, 6) for i in range(attacking_die)],
                                 reverse=True)

        for i in range(min(defending_die, attacking_die)):
            if attacking_rolls[i] > defending_rolls[i]:
                country.troops -= 1
            else:
                self.troops -= 1
                attacking_troops -= 1  # Kept track in case of invasion

        if country.troops == 0:
            country.owner.countries.remove(country)
            self.owner.countries.add(country)
            country.owner = self.owner
            country.troops = attacking_troops + moving_troops
            self.troops -= (attacking_troops + moving_troops)
            return True
        return False

    def add_troops(self, owner, troops):
        assert owner
        assert owner == self.owner or (self.troops == 0 and self.owner is None)
        assert troops > 0

        if(self.owner is None):
            self.owner = owner
            self.owner.countries.add(self)
        self.troops += troops

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        return isinstance(other, Country) and self.name == other.name

    def __repr__(self):
        return '<Country %r>' % self.name

class Continent(object):
    def __init__(self, name, bonus):
        self.name = name
        self.countries = {}
        self.bonus = bonus

    def get_country_set(self):
        return set([country for country in self.countries])

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        return isinstance(other, Continent) and self.name == other.name

    def __repr__(self):
        return '<Continent %r>' % self.name

    def get_player_set(self):
        keys = self.countries.keys()
        return set([self.countries[key].owner for key in keys])


class Board(object):
    def __init__(self):
        self.continents = {}
        self.countries = {}
        self.cards = {}


class Card(object):
    def __init__(self, country_name, value):
        self.country_name = country_name
        self.value = value

    def is_set_with(self, card_two, card_three):
        assert card_two is not None
        assert card_three is not None
        wild_cards = [card for card in [self, card_two, card_three] if card.value == 'wild']
        return (len(wild_cards) >= 1) or (self.value == card_two.value == card_three.value) or (self.value != card_two.value != card_three.value)

    def __rerp__(self):
        return '<Card %r>' % self.country_name


class Player(object):

    def __init__(self, name):
        self.name = name
        self.cards = set()
        self.is_eliminated = False
        self.is_neutral = False
        self.countries = set()
        self.earned_card_this_turn = False
        self.troops_to_deploy = 0
        self.available_actions = []
        self.errors = 0

    def choose_country(self, country):
        assert country.owner is None
        country.add_troops(self,1)

    def deploy_troops(self, country, troops):
        country.add_troops(self,troops)

    def check_neutralized(self):
        if self.errors >= 3:
            self.is_neutral = True

    def has_card_set(self):
        combos = itertools.combinations(self.cards, 3)
        for potential_set in combos:
            if potential_set[0].is_set_with(potential_set[1], potential_set[2]):
                return True
        return False

    def has_attack(self):
        return bool({c for c in self.countries if c.troops > 1 and
                     {bc for bc in c.border_countries if bc.owner != self}})

    def has_reinforce(self):
        return bool({c for c in self.countries if c.troops > 1 and
                     {bc for bc in c.border_countries if bc.owner == self}})

    def get_country_choice(self):
        pass

    def get_card_spend(self):
        pass

    def get_troop_deployment(self, game):
        pass

    def reinforcement_order(self):
        pass

    def get_attack_order(self):
        pass

    def send_game(self, game):
        pass

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        return isinstance(other, Player) and self.name == other.name

    def __repr__(self):
        return '<Player %r>' % self.name


class Players(object):

    def __init__(self):
        self.players_list = []
        self.game_started = False

    def add_player(self, player):
        assert not self.game_started
        self.players_list.append(player)

    def start_game(self):
        assert 3 <= len(self.players_list) <= 6
        self.game_started = True
        random.shuffle(self.players_list)
        self.players_cycle = itertools.cycle(self.players_list)
        self.current_player = self.players_list[0]
        self.__generate_other_players()

    def __iter__(self):
        return (player for player in self.players_list)

    def __getitem__(self, key):
        return self.players_list[key]

    def __len__(self):
        return len(self.players_list)

    def __generate_other_players(self):
        self.other_players = [player for player in self.players_list if player is not self.current_player and not player.is_eliminated]

    def next(self):
        self.current_player = self.players_cycle.next()
        if self.current_player.is_eliminated:
            self.next()
        self.__generate_other_players()
        return self.current_player

    def restart(self):
        self.players_cycle = itertools.cycle(self.players_list)
        self.current_player = self.players_list[0]
        self.__generate_other_players()

    def choose_country(self, game):
        while not self.current_player.get_country_choice(game):
            self.broadcast_game(game)
        self.broadcast_game(game)

    def spend_cards(self, game):
        if self.current_player.has_card_set():
            while not self.current_player.get_card_spend(game):
               self.broadcast_game(game)
            self.broadcast_game(game)

    def force_cards_spend(self, game):
        assert len(self.current_player.cards) >= 5
        while not self.current_player.get_card_spend(game, force=True):
            self.broadcast_game(game)
        self.broadcast_game(game)

    def deploy_troops(self, game):
        while not self.current_player.get_troop_deployment(game):
            self.broadcast_game(game)
        self.broadcast_game(game)

    def attack(self, game):
        if self.current_player.has_attack():
            while not self.current_player.get_attack_order(game):
                self.broadcast_game(game)
            self.broadcast_game(game)

    def reinforce(self, game):
        if self.current_player.has_reinforce():
            while not self.current_player.get_reinforce_order(game):
                self.broadcast_game(game)
            self.broadcast_game(game)

    def broadcast_game(self, game):
        game.broadcast_count += 1
        [player.broadcast_game(game) for player in self.other_players]


def import_board_data(json_filename=None):
    if json_filename:
        board_file = open(json_filename)
    else:
        board_file = open(os.path.join(os.path.dirname(__file__), 'board_graph.json'))
    board_json = json.load(board_file)
    board_file.close()
    board = Board()
    countries = {}
    cards = {}
    #go through the json and create the list of countries
    for continent_name in board_json:
        board.continents[continent_name] = Continent(continent_name,board_json[continent_name]["bonus"])
        for country_name in board_json[continent_name]["countries"]:
            countries[country_name] = Country(country_name, board_json[continent_name]["countries"][country_name]["border countries"])
            cards[country_name] = (Card(country_name, board_json[continent_name]["countries"][country_name]["card"]))
            board.continents[continent_name].countries[country_name] = countries[country_name]
    #loop through the country list and replace all of the border country strings with references to that country
    for country_name in countries:
        borders = [countries[name] for name in countries[country_name].border_countries]
        countries[country_name].border_countries = borders
    board.countries = countries
    #add the two wild cards
    cards['wild1'] = (Card("wild1", "wild"))
    cards['wild2'] = (Card("wild2", "wild"))
    #add cards to board
    board.cards = cards
    #return a tuple with the board and the cards
    return board



"""
Risk
-----------------------

Objects and methods for playing the boardgame Risk
"""
from setuptools import setup, find_packages
import risk

setup(
        name = "Risk",
        version = risk.__version__,
        url = 'https://github.com/eriktaubeneck/hacker-risk',
        author = 'Erik Taubeneck',
        author_email = 'erik.taubeneck@gmail.com',
        description = 'Objects and mthods for playing the boardgame Risk',
        packages = find_packages(),
        package_data = {
            '':['*.json'],
            },
        zip_safe=False,
        platforms = 'any',
        include_package_data=True,
        test_suite="tests",
        install_requires=[],
        tests_require=[],
        classifiers=[
                    'Environment :: Web Environment',
                    'Intended Audience :: Developers',
                    'Operating System :: OS Independent',
                    'Programming Language :: Python',
                    'Topic :: Software Development :: Libraries :: Python Modules'
                    ],
    )



import game 
import unittest
import random
import models
from mock import patch
from models import Players

class TestGame(unittest.TestCase):
    def setUp(self):
        self.players = Players()
        self.players.add_player(models.Player('Erik'))
        self.players.add_player(models.Player('Erty'))
        self.players.add_player(models.Player('Alex'))
        self.game = game.Game(self.players)
        self.players.start_game()

    def test_deployment_phase(self):
        country_A = self.game.board.countries["alaska"]
        country_A.owner = self.players.current_player
        
        def side_effect(country, troops):
            country.add_troops(country.owner, troops)

        with patch.object(models.Player, "get_troop_deployment") as mock_method:
            mock_method.side_effect = side_effect(country_A, 5)
            self.game.deployment_phase()
            self.assertEqual(country_A.troops, 5)
            self.assertEqual(country_A.owner.troops_to_deploy, 3)

    def test_attacking_phase(self):
        country_A = self.game.board.countries["northwest territory"]
        country_B = self.game.board.countries["alaska"]
        country_A.owner = self.players.current_player
        country_B.owner = models.Player("bob")
        country_A.troops = 10
        country_B.troops = 5
        with patch.object(random, 'randint') as mock_random:
            mock_random.side_effect = [6,6,1,1,1]
            self.game.attack(country_A, country_B, 3, 3)
            self.assertEqual(country_A.troops, 8)
            self.assertEqual(country_B.troops, 5)

    def test_reinforce(self):
        country_A = self.game.board.countries["northwest territory"] 
        country_B = self.game.board.countries["alaska"]
        country_A.owner = self.players.current_player
        country_A.troops = 10
        country_B.troops = 5
        country_B.owner = self.players.current_player
        self.game.reinforce(country_A, country_B, 2)
        self.assertEqual(country_A.troops, 8)
        self.assertEqual(country_B.troops, 7)
    
    def test_check_winner(self):
        pass

    def test_elimination(self):
        alex = self.players[2]
        erty = self.players[1]
        self.game.eliminate_player(alex, erty)
        self.assertEqual(erty.is_eliminated, True)

    def test_trade_cards_for_troops(self):
        cards = [models.Card("country_A", "whiskey"), models.Card("country_B", "gin"), models.Card("country_C", "tequila")]
        self.players.current_player.cards = set(cards)
        trade_1_troops = self.game.get_troops_for_card_set(cards)
        self.assertEqual(trade_1_troops, 4)
        self.players.current_player.cards = set(cards)
        trade_2_troops = self.game.get_troops_for_card_set(cards)
        self.assertEqual(trade_2_troops, 6)
        cards2 = [models.Card("cowLand", "cow"), models.Card("mexico", "horse"), models.Card("canada", "horse")]
        self.players.current_player.cards = set(cards2)
        with self.assertRaises(AssertionError):
            trade_3_troops = self.game.get_troops_for_card_set(cards2)
    
    def test_init_deploy(self):
        pass

    def test_play_game(self):
        pass

    def test_start(self):
        pass
        #self.game.start_game()
